'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getActionsFromLocation = require('./getActionsFromLocation');

var _getActionsFromLocation2 = _interopRequireDefault(_getActionsFromLocation);

var _getLocationFromState = require('./getLocationFromState');

var _getLocationFromState2 = _interopRequireDefault(_getLocationFromState);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (routes, history) {
  var blockLocationChange = false;

  var storeListener = function storeListener(store) {
    return function () {
      var replace = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      if (!blockLocationChange) {
        var state = store.getState();
        var oldLocation = history.location;
        var newLocation = (0, _getLocationFromState2.default)(routes, oldLocation, state);

        if (!(0, _utils.locationsEqual)(oldLocation, newLocation)) {
          if (replace) {
            history.replace(newLocation);
          } else {
            history.push(newLocation);
          }
        }
      }
    };
  };

  var locationListener = function locationListener(store) {
    return function (location) {
      var state = store.getState();
      var actions = (0, _getActionsFromLocation2.default)(routes, location, state);

      blockLocationChange = true;
      actions.forEach(store.dispatch);
      blockLocationChange = false;

      storeListener(store)(true);
    };
  };

  var enhancer = function enhancer(createStore) {
    return function () {
      var store = createStore.apply(undefined, arguments);

      store.subscribe(storeListener(store));
      history.listen(locationListener(store));

      return store;
    };
  };

  var syncLocationWithStore = function syncLocationWithStore(store) {
    storeListener(store)(true);
  };

  var syncStoreWithLocation = function syncStoreWithLocation(store) {
    locationListener(store)(history.location);
  };

  return {
    enhancer: enhancer,
    syncLocationWithStore: syncLocationWithStore,
    syncStoreWithLocation: syncStoreWithLocation
  };
};